@@ -5,7 +5,7 @@
 # it under the terms of version 2 of the GNU General Public License as
 # published by the Free Software Foundation.
 #
-# $Id: id3.py 4026 2007-04-27 02:13:05Z piman $
+# $Id: id3.py 4275 2008-06-01 06:32:37Z piman $
 
 """ID3v2 reading and writing.
 
@@ -220,8 +220,15 @@ class ID3(DictProxy, mutagen.Metadata):
                 raise ValueError("'%s' has invalid flags %#02x" % (fn, flags))
 
         if self.f_extended:
-            self.__extsize = BitPaddedInt(self.__fullread(4))
-            self.__extdata = self.__fullread(self.__extsize - 4)
+            if self.version >= (2,4,0):
+                # "Where the 'Extended header size' is the size of the whole
+                # extended header, stored as a 32 bit synchsafe integer."
+                self.__extsize = BitPaddedInt(self.__fullread(4)) - 4
+            else:
+                # "Where the 'Extended header size', currently 6 or 10 bytes,
+                # excludes itself."
+                self.__extsize = unpack('>L', self.__fullread(4))[0]
+            self.__extdata = self.__fullread(self.__extsize)
 
     def __determine_bpi(self, data, frames):
         if self.version < (2,4,0): return int
@@ -317,7 +324,15 @@ class ID3(DictProxy, mutagen.Metadata):
         The lack of a way to update only an ID3v1 tag is intentional.
         """
 
-        framedata = map(self.__save_frame, self.values())
+        # Sort frames by 'importance'
+        order = ["TIT2", "TPE1", "TRCK", "TALB", "TPOS", "TDRC", "TCON"]
+        order = dict(zip(order, range(len(order))))
+        last = len(order)
+        frames = self.items()
+        frames.sort(lambda a, b: cmp(order.get(a[0][:4], last),
+                                     order.get(b[0][:4], last)))
+
+        framedata = [self.__save_frame(frame) for (key, frame) in frames]
         framedata.extend([data for data in self.unknown_frames
                 if len(data) > 10])
         if not framedata:
@@ -416,27 +431,35 @@ class ID3(DictProxy, mutagen.Metadata):
         # unsafe to write
 
         # TDAT, TYER, and TIME have been turned into TDRC.
-        if str(self.get("TYER", "")).strip("\x00"):
-            date = str(self.pop("TYER"))
-            if str(self.get("TDAT", "")).strip("\x00"):
-                dat = str(self.pop("TDAT"))
-                date = "%s-%s-%s" % (date, dat[2:], dat[:2])
-                if str(self.get("TIME", "")).strip("\x00"):
-                    time = str(self.pop("TIME"))
-                    date += "T%s:%s:00" % (time[:2], time[2:])
-            if "TDRC" not in self:
-                self.add(TDRC(encoding=0, text=date))
+        try:
+            if str(self.get("TYER", "")).strip("\x00"):
+                date = str(self.pop("TYER"))
+                if str(self.get("TDAT", "")).strip("\x00"):
+                    dat = str(self.pop("TDAT"))
+                    date = "%s-%s-%s" % (date, dat[2:], dat[:2])
+                    if str(self.get("TIME", "")).strip("\x00"):
+                        time = str(self.pop("TIME"))
+                        date += "T%s:%s:00" % (time[:2], time[2:])
+                if "TDRC" not in self:
+                    self.add(TDRC(encoding=0, text=date))
+        except UnicodeDecodeError:
+            # Old ID3 tags have *lots* of Unicode problems, so if TYER
+            # is bad, just chuck the frames.
+            pass
 
         # TORY can be the first part of a TDOR.
         if "TORY" in self:
-            date = str(self.pop("TORY"))
+            f = self.pop("TORY")
             if "TDOR" not in self:
-                self.add(TDOR(encoding=0, text=date))
+                try:
+                    self.add(TDOR(encoding=0, text=str(f)))
+                except UnicodeDecodeError:
+                    pass
 
         # IPLS is now TIPL.
         if "IPLS" in self:
+            f = self.pop("IPLS")
             if "TIPL" not in self:
-                f = self.pop("IPLS")
                 self.add(TIPL(encoding=f.encoding, people=f.people))
 
         if "TCON" in self:
@@ -507,14 +530,14 @@ class BitPaddedInt(int):
         numeric_value = 0
         for shift, byte in zip(range(0, len(bytes)*bits, bits), bytes):
             numeric_value += byte << shift
-        return super(BitPaddedInt, cls).__new__(cls, numeric_value)
-
-    def __init__(self, value, bits=7, bigendian=True):
-        "Strips 8-bits bits out of every byte"
+        if isinstance(numeric_value, long):
+            self = long.__new__(BitPaddedLong, numeric_value)
+        else:
+            self = int.__new__(BitPaddedInt, numeric_value)
         self.bits = bits
         self.bigendian = bigendian
-        super(BitPaddedInt, self).__init__(value)
-    
+        return self
+
     def as_str(value, bits=7, bigendian=True, width=4):
         bits = getattr(value, 'bits', bits)
         bigendian = getattr(value, 'bigendian', bigendian)
@@ -533,6 +556,11 @@ class BitPaddedInt(int):
         return ''.join(map(chr, bytes))
     to_str = staticmethod(as_str)
 
+class BitPaddedLong(long):
+    def as_str(value, bits=7, bigendian=True, width=4):
+        return BitPaddedInt.to_str(value, bits, bigendian, width)
+    to_str = staticmethod(as_str)
+
 class unsynch(object):
     def decode(value):
         output = []
@@ -543,7 +571,7 @@ class unsynch(object):
                 append(val)
                 safe = val != '\xFF'
             else:
-                if val > '\xE0': raise ValueError('invalid sync-safe string')
+                if val >= '\xE0': raise ValueError('invalid sync-safe string')
                 elif val != '\x00': append(val)
                 safe = True
         if not safe: raise ValueError('string ended unsafe')
@@ -785,6 +813,8 @@ class VolumePeakSpec(Spec):
         peak = 0
         bits = ord(data[0])
         bytes = min(4, (bits + 7) >> 3)
+        # not enough frame data
+        if bytes + 1 > len(data): raise ID3JunkFrameError
         shift = ((8 - (bits & 7)) & 7) + (4 - bytes) * 8
         for i in range(1, bytes+1):
             peak *= 256
@@ -1224,6 +1254,7 @@ class TCON(TextFrame):
         return " / ".join(self.genres)
 
 class TCOP(TextFrame): "Copyright (c)"
+class TCMP(NumericTextFrame): "iTunes Compilation Flag"
 class TDAT(TextFrame): "Date of recording (DDMM)"
 class TDEN(TimeStampTextFrame): "Encoding Time"
 class TDOR(TimeStampTextFrame): "Original Release Time"
@@ -1747,6 +1778,7 @@ class TRD(TRDA): "Recording Dates"
 class TMT(TMED): "Source Media Type"
 class TFT(TFLT): "File Type"
 class TBP(TBPM): "Beats per minute"
+class TCP(TCMP): "iTunes Compilation Flag"
 class TCR(TCOP): "Copyright (C)"
 class TPB(TPUB): "Publisher"
 class TEN(TENC): "Encoder"
