@@ -4,7 +4,7 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation.
 #
-# $Id: m4a.py 4227 2007-12-04 07:41:03Z piman $
+# $Id: m4a.py 4275 2008-06-01 06:32:37Z piman $
 
 """Read and write MPEG-4 audio files with iTunes metadata.
 
@@ -95,10 +95,12 @@ class Atom(object):
 
     def render(name, data):
         """Render raw atom data."""
-        try:
-            return struct.pack(">I4s", len(data) + 8, name) + data
-        except OverflowError:
-            return struct.pack(">I4sQ", 1, name, len(data) + 16) + data
+        # this raises OverflowError if Py_ssize_t can't handle the atom data
+        size = len(data) + 8
+        if size <= 0xFFFFFFFF:
+            return struct.pack(">I4s", size, name) + data
+        else:
+            return struct.pack(">I4sQ", 1, name, size + 8) + data
     render = staticmethod(render)
 
     def __getitem__(self, remaining):
