@@ -4,7 +4,7 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation.
 #
-# $Id: mp4.py 4227 2007-12-04 07:41:03Z piman $
+# $Id: mp4.py 4275 2008-06-01 06:32:37Z piman $
 
 """Read and write MPEG-4 audio files with iTunes metadata.
 
@@ -87,10 +87,12 @@ class Atom(object):
 
     def render(name, data):
         """Render raw atom data."""
-        try:
-            return struct.pack(">I4s", len(data) + 8, name) + data
-        except OverflowError:
-            return struct.pack(">I4sQ", 1, name, len(data) + 16) + data
+        # this raises OverflowError if Py_ssize_t can't handle the atom data
+        size = len(data) + 8
+        if size <= 0xFFFFFFFF:
+            return struct.pack(">I4s", size, name) + data
+        else:
+            return struct.pack(">I4sQ", 1, name, size + 8) + data
     render = staticmethod(render)
 
     def findall(self, name, recursive=False):
@@ -462,7 +464,7 @@ class MP4Tags(DictProxy, Metadata):
         # Translate to a freeform genre.
         genre = cdata.short_be(data[16:18])
         if "\xa9gen" not in self:
-            try: self["\xa9gen"] = GENRES[genre - 1]
+            try: self["\xa9gen"] = [GENRES[genre - 1]]
             except IndexError: pass
 
     def __parse_tempo(self, atom, data):
