@@ -197,4 +197,11 @@ class TVCommentDict(TestCase):
         self.failUnlessEqual(d["artist"], self.c["artist"])
         self.failUnlessEqual(d["title"], self.c["title"])
 
+    def test_bad_key(self):
+        self.failUnlessRaises(UnicodeError, self.c.get, u"\u1234")
+        self.failUnlessRaises(
+            UnicodeError, self.c.__setitem__, u"\u1234", "foo")
+        self.failUnlessRaises(
+            UnicodeError, self.c.__delitem__, u"\u1234")
+
 add(TVCommentDict)
