@@ -128,9 +128,18 @@ class ID3Loading(TestCase):
         id3._ID3__fileobj = StringIO(
             'ID3\x04\x00\x40\x00\x00\x00\x00\x00\x00\x00\x05\x5a')
         id3._ID3__load_header()
-        self.assertEquals(id3._ID3__extsize, 5)
+        self.assertEquals(id3._ID3__extsize, 1)
         self.assertEquals(id3._ID3__extdata, '\x5a')
 
+    def test_header_2_3_extended(self):
+        id3 = ID3()
+        id3._ID3__fileobj = StringIO(
+            'ID3\x03\x00\x40\x00\x00\x00\x00\x00\x00\x00\x06'
+            '\x00\x00\x56\x78\x9a\xbc')
+        id3._ID3__load_header()
+        self.assertEquals(id3._ID3__extsize, 6)
+        self.assertEquals(id3._ID3__extdata, '\x00\x00\x56\x78\x9a\xbc')
+
     def test_unsynch(self):
         id3 = ID3()
         id3.version = (2,4,0)
@@ -766,6 +775,17 @@ class BitPaddedIntTest(TestCase):
     def test_65(self):
         self.assertEquals(BitPaddedInt('\x00\x00\x01\x81', 6), 0x41)
 
+    def test_32b(self):
+        self.assertEquals(BitPaddedInt('\xFF\xFF\xFF\xFF', bits=8),
+            0xFFFFFFFF)
+
+    def test_32bi(self):
+        self.assertEquals(BitPaddedInt(0xFFFFFFFF, bits=8), 0xFFFFFFFF)
+
+    def test_s32b(self):
+        self.assertEquals(BitPaddedInt('\xFF\xFF\xFF\xFF', bits=8).as_str(),
+            '\xFF\xFF\xFF\xFF')
+
     def test_s0(self):
         self.assertEquals(BitPaddedInt.to_str(0), '\x00\x00\x00\x00')
 
@@ -802,6 +822,7 @@ class BitPaddedIntTest(TestCase):
         self.assertEquals(len(BitPaddedInt.to_str(100, width=-1)), 4)
         self.assertEquals(len(BitPaddedInt.to_str(2**32, width=-1)), 5)
 
+
 class SpecSanityChecks(TestCase):
     uses_mmap = False
 
@@ -959,6 +980,7 @@ class FrameSanityChecks(TestCase):
         from mutagen.id3 import unsynch as un
         self.assertRaises(ValueError, un.decode, '\xff\xff\xff\xff')
         self.assertRaises(ValueError, un.decode, '\xff\xf0\x0f\x00')
+        self.assertRaises(ValueError, un.decode, '\xff\xe0')
         self.assertEquals('\xff\x44', un.decode('\xff\x44'))
 
     def test_load_write(self):
@@ -1172,6 +1194,11 @@ class BrokenDiscarded(TestCase):
         data = '\x01{\xf0\x10\xff\xff\x00'
         self.assertRaises(ID3JunkFrameError, RVA2.fromData, _24, 0x00, data)
 
+    def test_bad_number_of_bits_RVA2(self):
+        from mutagen.id3 import RVA2, ID3JunkFrameError
+        data = '\x00\x00\x01\xe6\xfc\x10{\xd7'
+        self.assertRaises(ID3JunkFrameError, RVA2.fromData, _24, 0x00, data)
+
     def test_drops_truncated_frames(self):
         from mutagen.id3 import Frames
         id3 = ID3()
@@ -1572,6 +1599,37 @@ class BadTYER(TestCase):
     def tearDown(self):
         del(self.audio)
 
+class BadPOPM(TestCase):
+    uses_mmap = False
+    filename = join('tests', 'data', 'bad-POPM-frame.mp3')
+    newfilename = join('tests', 'data', 'bad-POPM-frame-written.mp3')
+
+    def setUp(self):
+        shutil.copy(self.filename, self.newfilename)
+
+    def tearDown(self):
+        try: os.unlink(self.newfilename)
+        except EnvironmentError: pass
+
+    def test_read_popm_long_counter(self):
+        f = ID3(self.newfilename)
+        self.failUnless("POPM:Windows Media Player 9 Series" in f)
+        popm = f["POPM:Windows Media Player 9 Series"]
+        self.assertEquals(popm.rating, 255)
+        self.assertEquals(popm.count, 2709193061)
+
+    def test_write_popm_long_counter(self):
+        from mutagen.id3 import POPM
+        f = ID3(self.newfilename)
+        f.add(POPM(email="foo@example.com", rating=125, count=2**32+1))
+        f.save()
+        f = ID3(self.newfilename)
+        self.failUnless("POPM:foo@example.com" in f)
+        self.failUnless("POPM:Windows Media Player 9 Series" in f)
+        popm = f["POPM:foo@example.com"]
+        self.assertEquals(popm.rating, 125)
+        self.assertEquals(popm.count, 2**32+1)
+
 add(ID3Loading)
 add(ID3GetSetDel)
 add(BitPaddedIntTest)
@@ -1588,6 +1646,7 @@ add(OddWrites)
 add(NoHash)
 add(FrameIDValidate)
 add(BadTYER)
+add(BadPOPM)
 
 try: import eyeD3
 except ImportError: pass
